@@ -5,6 +5,9 @@
 */
 
 #include "kosmindoormapquickplugin.h"
+
+#include "amenitymodel.h"
+#include "amenitysortfilterproxymodel.h"
 #include "floorlevelchangemodel.h"
 #include "mapitem.h"
 #include "osmaddress.h"
@@ -21,16 +24,18 @@ using namespace KOSMIndoorMap;
 void KOSMIndoorMapQuickPlugin::registerTypes(const char *uri)
 {
     Q_UNUSED(uri);
+    Q_INIT_RESOURCE(assets);
+
+    qRegisterMetaType<MapData>();
     qRegisterMetaType<OSMAddress>();
     qRegisterMetaType<OSMElement>();
     qRegisterMetaType<Platform>();
     qRegisterMetaType<Platform::Mode>();
 
-    qmlRegisterUncreatableType<MapData>("org.kde.kosmindoormap", 1, 0, "MapData", {});
-    qmlRegisterUncreatableType<OSMAddress>("org.kde.kosmindoormap", 1, 0, "OSMAddress", {});
-    qmlRegisterUncreatableType<OSMElement>("org.kde.kosmindoormap", 1, 0, "OSMElement", {});
-    qmlRegisterUncreatableType<Platform>("org.kde.kosmindoormap", 1, 0, "Platform", {});
+    qmlRegisterUncreatableMetaObject(Platform::staticMetaObject, "org.kde.kosmindoormap", 1, 0, "Platform", {});
 
+    qmlRegisterType<AmenityModel>("org.kde.kosmindoormap", 1, 0, "AmenityModel");
+    qmlRegisterType<AmenitySortFilterProxyModel>("org.kde.kosmindoormap", 1, 0, "AmenitySortFilterProxyModel");
     qmlRegisterType<FloorLevelChangeModel>("org.kde.kosmindoormap", 1, 0, "FloorLevelChangeModel");
     qmlRegisterType<MapItem>("org.kde.kosmindoormap", 1, 0, "MapItemImpl");
     qmlRegisterType<OSMElementInformationModel>("org.kde.kosmindoormap", 1, 0, "OSMElementInformationModel");
