@@ -5,8 +5,9 @@
 */
 
 #include "osmelementinformationmodel.h"
-#include "osmelementinformationmodel_p.h"
 #include "osmelementinformationmodel_data.cpp"
+
+#include "localization.h"
 #include "osmaddress.h"
 
 #include <KLocalizedString>
@@ -42,6 +43,7 @@ bool OSMElementInformationModel::Info::operator==(OSMElementInformationModel::In
 
 OSMElementInformationModel::OSMElementInformationModel(QObject *parent)
     : QAbstractListModel(parent)
+    , m_langs(OSM::Languages::fromQLocale(QLocale()))
 {
 }
 
@@ -546,10 +548,10 @@ QVariant OSMElementInformationModel::valueForKey(Info info) const
     switch (info.key) {
         case NoKey: return {};
         case Name: {
-            const auto n = QString::fromUtf8(m_element.tagValue(QLocale(), "name", "loc_name", "int_name", "brand", "ref", "species", "genus"));
+            const auto n = QString::fromUtf8(m_element.tagValue(m_langs, "name", "loc_name", "int_name", "brand", "ref", "species", "genus"));
             const auto script = scriptForString(n);
             if (!isSameScript(QLocale().script(), script) && script > QChar::Script_Latin) {
-                const auto transliterated = QString::fromUtf8(m_element.tagValue(QLocale(), "int_name"));
+                const auto transliterated = QString::fromUtf8(m_element.tagValue(m_langs, "int_name"));
                 if (transliterated.isEmpty() || transliterated == n) {
                     return n;
                 }
@@ -585,13 +587,13 @@ QVariant OSMElementInformationModel::valueForKey(Info info) const
                 if ((*it).isEmpty() || (*it) == "yes" || (*it) == "no" || (*it) == "vending_machine" || (*it) == "building") {
                     continue;
                 }
-                out.push_back(translateValue((*it).constData(), amenity_map));
+                out.push_back(Localization::amenityType((*it).constData()));
             }
 
             if (out.isEmpty()) { // fall back to building, but only take terms we have translated
                 appendNonEmpty(m_element.tagValue("building"), l);
                 for (const auto &key : l) {
-                    auto s = translateValue(key.constData(), amenity_map, ReturnEmptyOnUnknownKey);
+                    auto s = Localization::amenityType(key.constData(), Localization::ReturnEmptyOnUnknownKey);
                     if (!s.isEmpty()) {
                         out.push_back(std::move(s));
                     }
@@ -608,7 +610,7 @@ QVariant OSMElementInformationModel::valueForKey(Info info) const
             return l.join(QLatin1String(", "));
         }
         case Description:
-            return m_element.tagValue(QLocale(), "description");
+            return m_element.tagValue(m_langs, "description");
         case Routes:
         {
             auto l = QString::fromUtf8(m_element.tagValue("route_ref", "bus_routes", "bus_lines", "buses")).split(QLatin1Char(';'), Qt::SkipEmptyParts);
@@ -617,7 +619,7 @@ QVariant OSMElementInformationModel::valueForKey(Info info) const
             }
             return QLocale().createSeparatedList(l);
         }
-        case Cuisine: return translateValues(m_element.tagValue("cuisine"), cuisine_map);
+        case Cuisine: return Localization::cuisineTypes(m_element.tagValue("cuisine"));
         case Diet:
         {
             QStringList l;
@@ -748,7 +750,7 @@ QVariant OSMElementInformationModel::valueForKey(Info info) const
             }
             return QLocale().createSeparatedList(l);
         }
-        case Wikipedia: return wikipediaUrl(m_element.tagValue(QLocale(), "wikipedia", "brand:wikipedia", "species:wikipedia"));
+        case Wikipedia: return wikipediaUrl(m_element.tagValue(m_langs, "wikipedia", "brand:wikipedia", "species:wikipedia"));
         case Address: return QVariant::fromValue(OSMAddress(m_element));
         case Phone: return QString::fromUtf8(m_element.tagValue("contact:phone", "phone", "telephone", "operator:phone"));
         case Email: return QString::fromUtf8(m_element.tagValue("contact:email", "email", "operator:email"));
@@ -785,7 +787,7 @@ QVariant OSMElementInformationModel::valueForKey(Info info) const
                 wheelchair = m_element.tagValue("wheelchair");
             }
             const auto a = translateValue(wheelchair.constData(), wheelchair_map);
-            const auto d = QString::fromUtf8(m_element.tagValue(QLocale(), "wheelchair:description"));
+            const auto d = QString::fromUtf8(m_element.tagValue(m_langs, "wheelchair:description"));
             if (!d.isEmpty()) {
                 return QString(a + QLatin1String(" (") + d + QLatin1Char(')'));
             }
@@ -798,14 +800,14 @@ QVariant OSMElementInformationModel::valueForKey(Info info) const
             return QString::fromUtf8(m_element.tagValue("centralkey"));
         case SpeechOutput:
             // TODO: rather than as a boolean value, list the available languages here when we have that information
-            return translatedBoolValue(m_element.tagValue(QLocale(), "speech_output"));
+            return translatedBoolValue(m_element.tagValue(m_langs, "speech_output"));
         case TactileWriting:
         {
             // TODO: rather than as a boolean value, list the available languages here when we have that information
             QStringList l;
             bool explicitNo = false;
             for (const auto &writing : tactile_writing_map) {
-                const auto v = m_element.tagValue(QLocale(), writing.keyName);
+                const auto v = m_element.tagValue(m_langs, writing.keyName);
                 if (v.isEmpty()) {
                     continue;
                 }
@@ -818,7 +820,7 @@ QVariant OSMElementInformationModel::valueForKey(Info info) const
             if (!l.isEmpty()) {
                 return QLocale().createSeparatedList(l);
             }
-            const auto v = m_element.tagValue(QLocale(), "tactile_writing");
+            const auto v = m_element.tagValue(m_langs, "tactile_writing");
             if (explicitNo && v.isEmpty()) {
                 return i18n("no");
             }
@@ -826,7 +828,7 @@ QVariant OSMElementInformationModel::valueForKey(Info info) const
         }
         case OperatorName: return QString::fromUtf8(m_element.tagValue("operator"));
         case Network: return QString::fromUtf8(m_element.tagValue("network"));
-        case OperatorWikipedia: return wikipediaUrl(m_element.tagValue(QLocale(), "operator:wikipedia", "network:wikipedia"));
+        case OperatorWikipedia: return wikipediaUrl(m_element.tagValue(m_langs, "operator:wikipedia", "network:wikipedia"));
         case RemainingRange:
         {
             const auto range = m_element.tagValue("mx:remaining_range").toInt();
