@@ -13,6 +13,7 @@
 #include "poleofinaccessibilityfinder_p.h"
 #include "scenegeometry_p.h"
 #include "openinghourscache_p.h"
+#include "texturecache_p.h"
 #include "../style/mapcssdeclaration_p.h"
 #include "../style/mapcssstate_p.h"
 #include "../style/mapcssresult_p.h"
@@ -45,12 +46,14 @@ public:
     QColor m_defaultTextColor;
     QFont m_defaultFont;
     QPolygonF m_labelPlacementPath;
+    TextureCache m_textureCache;
     IconLoader m_iconLoader;
     OpeningHoursCache m_openingHours;
     PoleOfInaccessibilityFinder m_piaFinder;
 
     OSM::TagKey m_layerTag;
     OSM::TagKey m_typeTag;
+    OSM::Languages m_langs;
 
     bool m_dirty = true;
     bool m_overlay = false;
@@ -59,7 +62,10 @@ public:
 
 using namespace KOSMIndoorMap;
 
-SceneController::SceneController() : d(new SceneControllerPrivate) {}
+SceneController::SceneController() : d(new SceneControllerPrivate)
+{
+    d->m_langs = OSM::Languages::fromQLocale(QLocale());
+}
 SceneController::~SceneController() = default;
 
 void SceneController::setMapData(const MapData &data)
@@ -221,6 +227,11 @@ void SceneController::updateElement(OSM::Element e, int level, SceneGraph &sg) c
     }
 }
 
+[[nodiscard]] static bool canWordWrap(const QString &s)
+{
+    return std::any_of(s.begin(), s.end(), [](QChar c) { return !c.isLetter(); });
+}
+
 void SceneController::updateElement(OSM::Element e, int level, SceneGraph &sg, const MapCSSResultItem &result) const
 {
     if (result.hasAreaProperties()) {
@@ -246,31 +257,48 @@ void SceneController::updateElement(OSM::Element e, int level, SceneGraph &sg, c
         }
 
         double lineOpacity = 1.0;
+        double casingOpacity = 1.0;
         double fillOpacity = 1.0;
+        bool hasTexture = false;
+        item->z = 0;
         initializePen(item->pen);
+        initializePen(item->casingPen);
         for (auto decl : result.declarations()) {
             applyGenericStyle(decl, item);
             applyPenStyle(e, decl, item->pen, lineOpacity, item->penWidthUnit);
+            applyCasingPenStyle(e, decl, item->casingPen, casingOpacity, item->casingPenWidthUnit);
             switch (decl->property()) {
                 case MapCSSDeclaration::FillColor:
-                    item->brush.setColor(decl->colorValue());
-                    item->brush.setStyle(Qt::SolidPattern);
+                    item->fillBrush.setColor(decl->colorValue());
+                    item->fillBrush.setStyle(Qt::SolidPattern);
                     break;
                 case MapCSSDeclaration::FillOpacity:
                     fillOpacity = decl->doubleValue();
                     break;
+                case MapCSSDeclaration::FillImage:
+                    item->textureBrush.setTextureImage(d->m_textureCache.image(decl->stringValue()));
+                    hasTexture = true;
+                    break;
                 default:
                     break;
             }
         }
         finalizePen(item->pen, lineOpacity);
-        if (item->brush.style() == Qt::SolidPattern && fillOpacity < 1.0) {
-            auto c = item->brush.color();
+        finalizePen(item->casingPen, casingOpacity);
+        if (item->fillBrush.style() == Qt::SolidPattern && item->textureBrush.style() == Qt::NoBrush && fillOpacity < 1.0) {
+            auto c = item->fillBrush.color();
             c.setAlphaF(c.alphaF() * fillOpacity);
-            item->brush.setColor(c);
+            item->fillBrush.setColor(c);
         }
-        if (item->brush.color().alphaF() == 0.0) {
-            item->brush.setStyle(Qt::NoBrush);
+        if (item->fillBrush.color().alphaF() == 0.0) {
+            item->fillBrush.setStyle(Qt::NoBrush);
+        }
+        if (hasTexture && item->textureBrush.style() != Qt::NoBrush && fillOpacity > 0.0) {
+            auto c = item->textureBrush.color();
+            c.setAlphaF(fillOpacity);
+            item->textureBrush.setColor(c);
+        } else {
+            item->textureBrush.setStyle(Qt::NoBrush);
         }
 
         addItem(sg, e, level, result, std::move(baseItem));
@@ -283,6 +311,7 @@ void SceneController::updateElement(OSM::Element e, int level, SceneGraph &sg, c
 
         double lineOpacity = 1.0;
         double casingOpacity = 1.0;
+        item->z = 0;
         initializePen(item->pen);
         initializePen(item->casingPen);
         for (auto decl : result.declarations()) {
@@ -306,7 +335,7 @@ void SceneController::updateElement(OSM::Element e, int level, SceneGraph &sg, c
 
         if (textDecl) {
             if (!textDecl->keyValue().isEmpty()) {
-                text = QString::fromUtf8(e.tagValue(QLocale(), textDecl->keyValue().constData()));
+                text = QString::fromUtf8(e.tagValue(d->m_langs, textDecl->keyValue().constData()));
             } else {
                 text = textDecl->stringValue();
             }
@@ -318,8 +347,13 @@ void SceneController::updateElement(OSM::Element e, int level, SceneGraph &sg, c
             auto baseItem = sg.findOrCreatePayload<LabelItem>(e, level, result.layerSelector());
             auto item = static_cast<LabelItem*>(baseItem.get());
             item->text.setText(text);
+            item->textIsSet = !text.isEmpty();
+            item->textOutputSizeCache = {};
             item->font = d->m_defaultFont;
             item->color = d->m_defaultTextColor;
+            item->iconSize = {};
+            item->textOffset = 0;
+            item->z = 0;
 
             double textOpacity = 1.0;
             double shieldOpacity = 1.0;
@@ -370,10 +404,14 @@ void SceneController::updateElement(OSM::Element e, int level, SceneGraph &sg, c
                         }
                         break;
                     case MapCSSDeclaration::TextOffset:
-                        item->offset = decl->doubleValue();
+                        item->textOffset = decl->doubleValue();
                         break;
                     case MapCSSDeclaration::MaxWidth:
-                        item->text.setTextWidth(decl->intValue());
+                        // work around for QStaticText misbehaving when we have a max width but can't actually word-wrap
+                        // far from perfect but covers the most common cases
+                        if (canWordWrap(text)) {
+                            item->text.setTextWidth(decl->intValue());
+                        }
                         break;
                     case MapCSSDeclaration::IconImage:
                         if (!decl->keyValue().isEmpty()) {
@@ -404,6 +442,12 @@ void SceneController::updateElement(OSM::Element e, int level, SceneGraph &sg, c
                     case MapCSSDeclaration::TextHaloRadius:
                         item->haloRadius = decl->doubleValue();
                         break;
+                    case MapCSSDeclaration::IconAllowIconOverlap:
+                        item->allowIconOverlap = decl->boolValue();
+                        break;
+                    case MapCSSDeclaration::IconAllowTextOverlap:
+                        item->allowTextOverlap = decl->boolValue();
+                        break;
                     default:
                         break;
                 }
@@ -459,7 +503,11 @@ void SceneController::updateElement(OSM::Element e, int level, SceneGraph &sg, c
                 opt.setAlignment(Qt::AlignHCenter);
                 opt.setWrapMode(item->text.textWidth() > 0.0 ? QTextOption::WordWrap : QTextOption::NoWrap);
                 item->text.setTextOption(opt);
-                item->text.prepare({}, item->font);
+                // do not use QStaticText::prepare here:
+                // the vast majority of text items will likely not be shown at all for being overlapped or out of view
+                // and pre-computing them is too expensive. Instead this will happen as needed on first use, for only
+                // a smaller amounts at a time.
+                // item->text.prepare({}, item->font);
 
                 // discard labels that are longer than the line they are aligned with
                 if (result.hasLineProperties() && d->m_labelPlacementPath.size() > 1 && item->angle != 0.0) {
@@ -473,6 +521,11 @@ void SceneController::updateElement(OSM::Element e, int level, SceneGraph &sg, c
                         item->text = {};
                     }
                 }
+
+                // put texts below icons by default
+                if (!item->icon.isNull() && item->textOffset == 0.0) {
+                    item->textOffset = item->iconSize.height(); // ### what about heights in meters?
+                }
             }
 
             if (!item->icon.isNull() || !item->text.text().isEmpty()) {
@@ -515,20 +568,23 @@ QPolygonF SceneController::createPolygon(OSM::Element e) const
 QPainterPath SceneController::createPath(const OSM::Element e, QPolygonF &outerPath) const
 {
     assert(e.type() == OSM::Type::Relation);
-    outerPath = createPolygon(e);
+    outerPath = createPolygon(e); // TODO this is actually not correct for the multiple outer polygon case
     QPainterPath path;
-    path.addPolygon(outerPath); // assemble the outer polygon, which can be represented as a set of unsorted lines here even
+    path.setFillRule(Qt::OddEvenFill);
 
     for (const auto &mem : e.relation()->members) {
         const bool isInner = std::strcmp(mem.role().name(), "inner") == 0;
-        if (mem.type() != OSM::Type::Way || !isInner) {
+        const bool isOuter = std::strcmp(mem.role().name(), "outer") == 0;
+        if (mem.type() != OSM::Type::Way || (!isInner && !isOuter)) {
             continue;
         }
         if (auto way = d->m_data.dataSet().way(mem.id)) {
             const auto subPoly = createPolygon(OSM::Element(way));
-            QPainterPath subPath;
-            subPath.addPolygon(subPoly);
-            path = path.subtracted(subPath);
+            if (subPoly.isEmpty()) {
+                continue;
+            }
+            path.addPolygon(subPoly);
+            path.closeSubpath();
         }
     }
 
@@ -630,6 +686,7 @@ void SceneController::applyFontStyle(const MapCSSDeclaration *decl, QFont &font)
 void SceneController::initializePen(QPen &pen) const
 {
     pen.setColor(Qt::transparent);
+    pen.setWidthF(0.0);
 
     // default according to spec
     pen.setCapStyle(Qt::FlatCap);
@@ -645,7 +702,7 @@ void SceneController::finalizePen(QPen &pen, double opacity) const
         pen.setColor(c);
     }
 
-    if (pen.color().alphaF() == 0.0) {
+    if (pen.color().alphaF() == 0.0 || pen.widthF() == 0.0) {
         pen.setStyle(Qt::NoPen); // so the renderer can skip this entirely
     }
 
