@@ -22,12 +22,12 @@ struct {
 } static constexpr const property_types[] = {
     // only those properties have their corresonding flag set that actually trigger emission of a scene graph item
     // e.g. for a label we either need a text or an icon, the visual properties for those on their own would be a no-op
-    { "casing-color", MapCSSDeclaration::CasingColor, MapCSSDeclaration::LabelProperty },
+    { "casing-color", MapCSSDeclaration::CasingColor, MapCSSDeclaration::NoFlag },
     { "casing-dashes", MapCSSDeclaration::CasingDashes, MapCSSDeclaration::NoFlag },
     { "casing-linecap", MapCSSDeclaration::CasingLineCap, MapCSSDeclaration::NoFlag },
     { "casing-linejoin", MapCSSDeclaration::CasingLineJoin, MapCSSDeclaration::NoFlag },
     { "casing-opacity", MapCSSDeclaration::CasingOpacity, MapCSSDeclaration::NoFlag },
-    { "casing-width", MapCSSDeclaration::CasingWidth, MapCSSDeclaration::LineProperty },
+    { "casing-width", MapCSSDeclaration::CasingWidth, MapCSSDeclaration::NoFlag },
     { "color", MapCSSDeclaration::Color, MapCSSDeclaration::LineProperty },
     { "dashes", MapCSSDeclaration::Dashes, MapCSSDeclaration::NoFlag },
     { "fill-color", MapCSSDeclaration::FillColor, MapCSSDeclaration::AreaProperty | MapCSSDeclaration::CanvasProperty }, // TODO this also applies to lines
@@ -38,6 +38,8 @@ struct {
     { "font-style", MapCSSDeclaration::FontStyle, MapCSSDeclaration::NoFlag },
     { "font-variant", MapCSSDeclaration::FontVariant, MapCSSDeclaration::NoFlag },
     { "font-weight", MapCSSDeclaration::FontWeight, MapCSSDeclaration::NoFlag },
+    { "icon-allow-icon-overlap", MapCSSDeclaration::IconAllowIconOverlap, MapCSSDeclaration::NoFlag },
+    { "icon-allow-text-overlap", MapCSSDeclaration::IconAllowTextOverlap, MapCSSDeclaration::NoFlag },
     { "icon-color", MapCSSDeclaration::IconColor, MapCSSDeclaration::NoFlag },
     { "icon-height", MapCSSDeclaration::IconHeight, MapCSSDeclaration::NoFlag },
     { "icon-image", MapCSSDeclaration::IconImage, MapCSSDeclaration::LabelProperty },
@@ -164,6 +166,11 @@ double MapCSSDeclaration::doubleValue() const
     return m_doubleValue;
 }
 
+bool MapCSSDeclaration::boolValue() const
+{
+    return m_boolValue;
+}
+
 QString MapCSSDeclaration::stringValue() const
 {
     return m_stringValue;
@@ -197,6 +204,11 @@ void MapCSSDeclaration::setDoubleValue(double val)
     m_doubleValue = val;
 }
 
+void MapCSSDeclaration::setBoolValue(bool val)
+{
+    m_boolValue = val;
+}
+
 void MapCSSDeclaration::setPropertyName(const char *name, std::size_t len)
 {
     const auto it = std::lower_bound(std::begin(property_types), std::end(property_types), name, [len](const auto &lhs, const char *rhs) {
@@ -351,12 +363,14 @@ void MapCSSDeclaration::write(QIODevice *out) const
                     out->write(QByteArray::number(d));
                     out->write(", ");
                 }
-            } else if (!m_stringValue.isEmpty()) {
+            } else if (!m_stringValue.isNull()) {
                 out->write("\"");
                 out->write(m_stringValue.toUtf8()); // this would need to be quoted...
                 out->write("\"");
-            } else {
+            } else if (!m_identValue.isEmpty()) {
                 out->write(m_identValue);
+            } else {
+                out->write(m_boolValue ? "true" : "false");
             }
 
             for (const auto &u : unit_map) {
