@@ -85,6 +85,8 @@ public:
         IconHeight, /// icon height
         IconOpacity, /// icon opacity
         IconColor, /// for colorized SVGs, non-standard extension
+        IconAllowTextOverlap, /// the equivalent to CartoCSS's allow-overlap, non-standard extension
+        IconAllowIconOverlap, /// the equivalent to CartoCSS's ignore-placement, non-standard extension
 
         // label properties
         FontFamily, /// font name
@@ -134,6 +136,7 @@ public:
     /** Numeric value for this property. */
     int intValue() const;
     double doubleValue() const;
+    bool boolValue() const;
     /** Quoted string value. */
     QString stringValue() const;
     /** Color value for this property. */
@@ -183,6 +186,7 @@ private:
     void setPropertyName(const char *name, std::size_t len);
     void setIdentifierValue(const char *val, int len);
     void setDoubleValue(double val);
+    void setBoolValue(bool val);
     void setStringValue(char *str);
     void setColorRgba(uint32_t argb);
     void setDashesValue(const QVector<double> &dashes);
@@ -201,6 +205,7 @@ private:
     ClassSelectorKey m_class;
     Unit m_unit = NoUnit;
     Type m_type;
+    bool m_boolValue = false;
 };
 
 }
